# coding: utf-8
import os
from . import paths
from configparser import ConfigParser

__all__ = ['getConfig']

defaultConfig = {
    'LOGGING.level': 'ERROR',
    'LOGIN.cookieFile': '.cookies',
    'LOGIN.envFile': '.env'
}

def getConfig(name, file='config.ini', default={}):
    '''返回能获取全局配置文件值的函数'''
    configPwd = os.path.join(paths.rootCwd, file)
    cfg = ConfigParser()
    cfg.read(configPwd, encoding='utf-8')
    def getval(key):
        if cfg.has_option(name, key):
            return cfg.get(name, key)
        return default.get(key) or defaultConfig.get('.'.join([name, key]))
    return getval


# coding: utf-8
import requests
from http.cookiejar import LWPCookieJar, CookieJar
from sunday.core import getConfig, Logger, Fetch, globalvar
from sunday.utils.tools import mergeObj
import os


class LoginBase():
    def __init__(self, file, logger=Logger('LoginBase').getLogger()):
        pwd = os.path.dirname(os.path.abspath(file))
        self.logger = logger
        cfg = getConfig('LOGIN')
        self.cookiePwd = os.path.join(pwd, cfg('cookieFile'))
        self.envPwd = os.path.join(pwd, cfg('envFile'))
        self.fetch = Fetch(pacWifiName = ['PA_WLAN'])

    def getCookiePwd(self):
        return self.cookiePwd

    def getEnvPwd(self):
        return self.envPwd

    def saveCookie(self):
        '''保存cookie'''
        self.fetch.session.cookies.save(ignore_discard=True, ignore_expires=True)

    def getCookie(self, useHistory = True):
        """ params(useHistory): 控制是否使用历史登陆态
            return(cookies, boolean): 返回cookie对象和是否登录标志
        """
        filename = self.getCookiePwd()
        hasCookieFile = os.path.isfile(filename)
        if useHistory and hasCookieFile:
            load_cookiejar = LWPCookieJar()
            load_cookiejar.load(filename, ignore_discard=True, ignore_expires=True)
            load_cookies = requests.utils.dict_from_cookiejar(load_cookiejar)
            self.logger.debug('是否存在登录态: 是')
            return requests.utils.cookiejar_from_dict(load_cookies), True
        self.logger.debug('是否存在登录态: 否')
        return LWPCookieJar(filename=filename), False

    def setCookie(self, name, value, domain, rest={}, **argvs):
        '''增加新的cookie, 入参参考cookielib.Cookie中的定义'''
        cookies = self.fetch.session.cookies
        standard = mergeObj({ 'path': '/', 'domain': domain, 'version': 0 }, argvs)
        cookie = CookieJar()._cookie_from_cookie_tuple((name, value, standard, rest), None)
        cookies.set_cookie(cookie)
    
    def checkLogin(self, checkUrl):
        '''确认是否登录成功, 方法: 请求首页, 如果跳到登录页则说明登录态失效'''
        # 服务端存在缓存因此请求两次
        self.fetch.get(checkUrl)
        res = self.fetch.get(checkUrl)
        return res.status_code != 200 or res.url != checkUrl

    def initRs(self, checkUrl):
        session = self.fetch.session
        session.cookies, isLogin = self.getCookie()
        if isLogin and self.checkLogin(checkUrl):
            session.cookies, isLogin = self.getCookie(useHistory = False)
        return self.fetch, isLogin




# coding: utf-8
import re
import subprocess

def checkWifi(name):
    '''传入wifi名, 返回当前wifi是否匹配'''
    if not name: return False
    if type(name) == str: name = [name]
    command = ['/System/Library/PrivateFrameworks/Apple80211.framework/Versions/Current/Resources/airport', '-I']
    res = subprocess.check_output(command).decode('utf-8')
    return any(re.search(re.compile('SSID: %s\n' % item), res) for item in name)



# coding: utf-8

# 工具函数
from .checkWifi import *
from .getConfig import *
from .getEnv import *
from .enver import *
from .printTable import *
from .getParser import *
from .common import *
from .cachevar import *
from .cmdexec import *
from .MultiThread import *
from . import globalvar

# 日志
from .logger import *

# 请求
from .fetch import *

# 目录
from . import paths



# coding: utf-8
import logging
import colorlog
from .getConfig import getConfig

class Logger():
    def __init__(self, name, level='', format=''):
        cfg = getConfig('LOGGING')
        logger = logging.getLogger(name)
        logger.setLevel(logging.getLevelName(level or cfg('level')))
        loghd = logging.StreamHandler()
        loghd.setFormatter(colorlog.ColoredFormatter(format or cfg('format'), datefmt='%H:%M:%S'))
        logger.addHandler(loghd)
        logger.propagate = True
        self.logger = logger
    
    def getLogger(self):
        return self.logger


# coding: utf-8
import requests
from http.cookiejar import LWPCookieJar
from .getConfig import getConfig
from .checkWifi import checkWifi
from .getEnv import getEnv
from .logger import Logger
from pypac import PACSession
from requests.auth import HTTPProxyAuth
import os, sys

# 关闭由verify=False引起的提示
requests.packages.urllib3.disable_warnings()

headers = {
    'User-Agent': ('Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_6) '
        'AppleWebKit/537.36 (KHTML, like Gecko) Chrome/70.0.3538.77 Safari/537.36'),
    'Pragma': 'no-cache',
    'Cache-Control': 'no-cache',
    'Accept-Encoding': 'gzip, deflate, br',
    'Accept-Language': 'zh-CN,zh;q=0.9'
}

headers_post = {
    'contentType': {
        'Content-Type': 'application/json;charset=UTF-8'
    }
}

logger = Logger('Fetch').getLogger()
class Fetch():
    def __init__(self, pacWifiName = None):
        self.pacWifiName = pacWifiName
        self._hasProxy = False
        self._hasPacProxy = False
        self.session = self.get_session()

    def get_session(self):
        # 返回session
        proxy = getConfig('PROXY')('proxy')
        session = requests.session() if proxy else PACSession()
        self.openProxy(session, proxy)
        session.headers.update(headers)
        session.verify = False
        return session
    
    def openProxy(self, session, proxy):
        '''开启代理'''
        if proxy:
            logger.warning('存在代理%s' % proxy)
            session.proxies = {
                'http': proxy,
                'https': proxy
            }
            self._hasProxy = True
        else:
            self.openPacProxy(session)
    
    def add_header(self, headers):
        '''更新请求头'''
        if type(headers) == dict:
            self.session.headers.update(headers)
    
    def closeProxy(self):
        '''关闭代理'''
        self.session.proxies = None
        self._hasProxy = False
    
    def hasProxy(self):
        '''返回是否开启代理'''
        return self._hasProxy

    def hasPacProxy(self):
        '''返回是否开启pac'''
        return self._hasPacProxy

    def openPacProxy(self, session):
        '''检测为内网环境则增加代理认证'''
        UM_USER = getEnv('UM_USER')
        UM_PASS = getEnv('UM_PASS')
        if UM_USER and UM_PASS and checkWifi(self.pacWifiName):
            logger.warning('pac认证网络%s' % self.pacWifiName)
            session.proxy_auth = HTTPProxyAuth(UM_USER, UM_PASS)
            self._hasPacProxy = True
            return True
        return False

    def requestByType(self, type, times, *args, **kwargs):
        try:
            res = getattr(self.session, type)(*args, **kwargs)
            logger.info('fetch result %s:%d' % (res.url, res.status_code))
            return res
        except Exception as e:
            if times >= 3:
                logger.error('第%d次尝试失败, 请检查网络后重试!' % times)
                sys.exit(1)
            times += 1
            logger.warning('接口请求失败, 进行第%d次尝试: %s' % (times, e))
            return self.requestByType(type, times, *args, **kwargs)
    
    def get(self, *args, **kwargs):
        return self.requestByType('get', 0, *args, **kwargs)

    def post(self, *args, **kwargs):
        return self.requestByType('post', 0, *args, **kwargs)
    
    def getCookiesDict(self):
        '''cookie转换成对象后返回'''
        cookies = self.session.cookies
        if 'get_dict' in cookies:
            return cookies.get_dict()
        return requests.utils.dict_from_cookiejar(cookies)

